/**
 * Ordered Set Drag 'n' Drop handler
 */

Drupal.behaviors.orderedSetDrag = function(context) {
  for (tableId in Drupal.settings.tableDrag) {
    var table = $('#' + tableId);

    var tableDrag = Drupal.tableDrag[tableId]; // Get the tableDrag object.

    // Add a handler for when a row is swapped, update empty categories.
    tableDrag.row.prototype.onSwap = function(swappedRow) {
      checkEmptyRegions(table, this);
    };

    Drupal.theme.tableDragChangedWarning = function() {
      return '<div class="warning">' + Drupal.theme('tableDragChangedMarker') + ' ' + Drupal.t("The changes made to this <em>Ordered Set</em> will not be saved until the <em>Save</em> button is clicked.") + '</div>';
    }


    // Add a handler so when a row is dropped, update fields dropped into new categories.
    tableDrag.onDrop = function() {
      dragObject = this;
      if ($(dragObject.rowObject.element).prev('tr').is('.region-message')) {
        var regionRow = $(dragObject.rowObject.element).prev('tr').get(0);
        var regionName = regionRow.className.replace(/([^ ]+[ ]+)*region-([^ ]+)-message([ ]+[^ ]+)*/, '$2');
        var visibleField = $('select.item-visible', dragObject.rowObject.element);
        var weightField = $('select.item-weight', dragObject.rowObject.element);
        var oldRegionName = weightField[0].className.replace(/([^ ]+[ ]+)*item-weight-([^ ]+)([ ]+[^ ]+)*/, '$2');

        if (!visibleField.is('.item-visible-'+ regionName)) {
          visibleField.removeClass('item-visible-' + oldRegionName).addClass('item-visible-' + regionName);
          weightField.removeClass('item-weight-' + oldRegionName).addClass('item-weight-' + regionName);
          visibleField.val(regionName);
        }
      }
    };


    $('select.item-visible:not(.item-visible-processed)', table).each(function() {
      $(this).change(function(event) {
        // Make our new row and select field.
        var row = $(this).parents('tr:first');
        var select = $(this);
        tableDrag.rowObject = new tableDrag.row(row);

        // Find the correct region and insert the row as the first in the region.
        $('tr.region-message', table).each(function() {
          if ($(this).is('.region-' + select[0].value + '-message')) {
            // Add the new row and remove the old one.
            $(this).after(row);
            // Manually update weights and restripe.
            tableDrag.updateFields(row.get(0));
            tableDrag.rowObject.changed = true;
            if (tableDrag.oldRowElement) {
              $(tableDrag.oldRowElement).removeClass('drag-previous');
            }
            tableDrag.oldRowElement = row.get(0);
            tableDrag.restripeTable();
            tableDrag.rowObject.markChanged();
            tableDrag.oldRowElement = row;
            $(row).addClass('drag-previous');
          }
        });

        // Modify empty regions with added or removed fields.
        checkEmptyRegions(table, row);
        // Remove focus from selectbox.
        select.get(0).blur();
      });
      $(this).addClass('item-visible-processed');
    });


    // Empty Region checking routine
    var checkEmptyRegions = function(table, rowObject) {
      $('tr.region-message', table).each(function() {
        // If the dragged row is in this region, but above the message row, swap it down one space.
        if ($(this).prev('tr').get(0) == rowObject.element) {
          // Prevent a recursion problem when using the keyboard to move rows up.
          if ((rowObject.method != 'keyboard' || rowObject.direction == 'down')) {
            rowObject.swap('after', this);
          }
        }
        // This region has become empty (the next row either isn't draggable OR there is no next row)
        if ($(this).next('tr').is(':not(.draggable)') || $(this).next('tr').size() == 0) {
          $(this).removeClass('region-populated').addClass('region-empty');
        }
        // This category has become populated (there IS a next row)
        else if ($(this).is('.region-empty')) {
          $(this).removeClass('region-empty').addClass('region-populated');
        }
      });
    };
  }
};
